// Copyright (C) INRIA 1999-2005
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 as published
// by the Free Software Foundation.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
// Public License for more details.
// 
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//%
// @file LagrangianModel/Bip/SomeDefinitions.scilab
// @author Pierre-Brice Wieber
//
// Affiliation(s): INRIA, team BIPOP
//
// Email(s): Pierre-Brice.Wieber@inria.fr
// 
// @brief Define the Lagrangian Model Name, the contacts names, the contact solids and
// the Stickfigure and the name of H-Anim Joints used in VRML Visualization
// 
 
// Description:
// 
// Modifications:
// $Log$
// Revision 1.6  2005/10/28 09:07:26  billet
// Bip Model VRML Visualization Modification in order to see the forces
//
// Revision 1.5  2005/07/25 16:15:58  billet
// Adding of VRML Visualization in LagrangianModel repertory in order to have a less complex tree structure
//
// Revision 1.4  2005/05/04 13:13:15  billet
// Correction and changes for doxygen documentation
//
// Revision 1.3  2005/05/03 11:45:18  billet
// Comments for doxygen documentation
//
// Revision 1.2  2005/04/07 11:41:30  wieber
// Added the variables ContactSolids (for use in Visu) and LagrangianModelName (for later use).
//
// Revision 1.1.1.1  2005/02/08 13:05:30  rpissard
// version start HuMAnS
//
// 


//Definitions for Lagrangian Model

LagrangianModelName = 'Bip_entier';

ContactNames = ['right exterior toe';...
		'right interior toe';...
		'right exterior heel';...
		'right interior heel';...
		'right ankle';...
		'right knee';...
		'right hip';...
		'left hip';...
		'left knee';...
		'left ankle';...
		'left exterior toe';...
		'left interior toe';...
		'left exterior heel';...
		'left interior heel';...
		'lombaires';...
		'down right trunk';...
		'down left trunk';...
		'up right trunk';...
		'up left trunk';...
		'back right suspension';...
		'back left suspension';...
		'front left suspension';...
		'front right suspension'];

ContactSolids = [1, 4;...
		 5, 5;...
		 6, 6;...
		 7, 8;...
		 9, 9;...
		 10, 10;...
		 11, 14;...
		 15, 23];
		 

StickFigure = [1, 2;...
	       2, 4;...
	       4, 3;...
	       3, 1;...
	       1, 5;...
	       2, 5;...
	       3, 5;...
	       4, 5;...
	       5, 6;...
	       6, 7;...
	       7, 8;...
	       8, 9;...
	       9, 10;...
	       10, 11;...
	       10, 12;...
	       10, 13;...
	       10, 14;...
	       11, 12;...
	       12, 14;...
	       14, 13;...
	       13, 11;...
	       7, 15;...
	       8, 15;...
	       15, 16;...
	       16, 18;...
	       18, 19;...
	       19, 17;...
	       17, 15];



//Definitions for VRML Visualization

// Names of the HAnim Joints

HAnimJointsNames = ['BipHigh_HumanoidRoot';...
		    'BipHigh_r_hipFrontal';...
		    'BipHigh_r_hipHorizontal';...
		    'BipHigh_r_hip';...
		    'BipHigh_r_knee';...
		    'BipHigh_r_ankleSagittal';...
		    'BipHigh_r_ankle';...
		    'BipHigh_l_hipFrontal';...
		    'BipHigh_l_hipHorizontal';...
		    'BipHigh_l_hip';...
		    'BipHigh_l_knee';...
		    'BipHigh_l_ankleSagittal';...
		    'BipHigh_l_ankle';...
		    'BipHigh_vl5Horizontal';...
		    'BipHigh_vl5Frontal';...
		    'BipHigh_vl5'];

//Names of the contact solids

HAnimContactSolidsNames = ['BipHigh_RightFoot';...
                           'BipHigh_RightForeLeg';...
                           'BipHigh_RightTight';...
			   'BipHigh_Thights';...
                           'BipHigh_LeftTight';...
                           'BipHigh_LeftForeLeg';...
			   'BipHigh_LeftFoot';...
				'BipHigh_Suspension';
				'BipHigh_Total'];
				
HAnimContactSolidsRGB = [[255; 0; 0],...
			[255; 0; 0],...
			[255; 0; 0],...
			[255; 0; 0],...
			[255; 0; 0],...
			[255; 0; 0],...
			[255; 0; 0],...
			[255; 0; 0],...
			[0; 0; 255]];				
				
				
				
				
				
				
				
