#include <math.h>

#include "robotmodel.h"

void modele_inertie(const double Q[N_DOF], double M[N_DOF*N_DOF])
{
  /* Intermediate cos and sin values */
  double C1, C2, C3, C4, C5, C6, C7;
  double C11, C22, C33, C44, C55, C66, C77;
  double S1, S2, S3, S4, S5, S6, S7;
  double S11, S22, S33, S44, S55, S66, S77;
  double CC1, CC2, CC3, CC4, CC5, CC6, CC7;
  double SS1, SS2, SS3, SS4, SS5, SS6, SS7;


  C1 = cos(Q[0]);
  C2 = cos(Q[1]);
  C3 = cos(Q[2]);
  C4 = cos(Q[3]);
  C5 = cos(Q[4]);
  C6 = cos(Q[5]);
  C7 = cos(Q[6]);

  C11 = cos(2 * Q[0]);
  C22 = cos(2 * Q[1]);
  C33 = cos(2 * Q[2]);
  C44 = cos(2 * Q[3]);
  C55 = cos(2 * Q[4]);
  C66 = cos(2 * Q[5]);
  C77 = cos(2 * Q[6]);

  S1 = sin(Q[0]);
  S2 = sin(Q[1]);
  S3 = sin(Q[2]);
  S4 = sin(Q[3]);
  S5 = sin(Q[4]);
  S6 = sin(Q[5]);
  S7 = sin(Q[6]);

  S11 = sin(2 * Q[0]);
  S22 = sin(2 * Q[1]);
  S33 = sin(2 * Q[2]);
  S44 = sin(2 * Q[3]);
  S55 = sin(2 * Q[4]);
  S66 = sin(2 * Q[5]);
  S77 = sin(2 * Q[6]);

  CC1 = pow(C1, 2);
  CC2 = pow(C2, 2);
  CC3 = pow(C3, 2);
  CC4 = pow(C4, 2);
  CC5 = pow(C5, 2);
  CC6 = pow(C6, 2);
  CC7 = pow(C7, 2);

  SS1 = pow(S1, 2);
  SS2 = pow(S2, 2);
  SS3 = pow(S3, 2);
  SS4 = pow(S4, 2);
  SS5 = pow(S5, 2);
  SS6 = pow(S6, 2);
  SS7 = pow(S7, 2);

  M[0] = Im1 + I18 + (-CC3 + CC2 * (1 + CC3)) * CC4 * I11 + (1 + (-CC3 + CC2 * (1 + CC3)) * CC4) * CC5 * I12 +  CC2 * I21 + \
         CC2 * CC5 * I22 + (\
                            1 - CC5 + CC3 * (-1 + CC4 * (1 + CC5)) + \
                            CC2 * (CC5 - CC4 * (1 + CC5) + CC3 * (1 - CC4 * (1 + CC5)))) * CC6*\
         I9 + (C2 * C3 * I20 * S2 * S44) / 2. + \
         CC5 * I10 * (CC3 - CC2 * CC3 + C2 * C3 * S2 * S44) + (\
             I5 * (C4 * S33 - C4 * CC2 * S33 + 2 * C2 * S2 * S3 * S4) * S55) / 8. + \
         I24 * (2 * C4 * C6 - 2 * C5 * S4 * S6 - S22 * S3 * S5 * S6 + \
                C2 * C3 * S2 * (2 * C6 * S4 + 2 * C4 * C5 * S6) + CC2 * (-2 * C4 * C6 + 2 * C5 * S4 * S6)) + \
         CC3 * I19 * SS2 + 2 * I23 * (C2 * C3 * S2 * S4 + C4 * SS2) + \
         I6 * (2 * C6 + (C5 * (C3 * C44 * S22 - 2 * C4 * S4 + S44) + 2 * C3 * S3 * S4 * S5) * S6 + \
               C2 * S2 * (2 * C3 * C6 * S44 - 2 * C4 * S3 * S5 * S6) + \
               CC3 * (-(C5 * S44 * S6) - 2 * C6 * SS4) + \
               CC2 * (-2 * C6 * CC4 + 2 * S4 * (C4 * C5 - C3 * S3 * S5) * S6 + \
                      CC3 * (C5 * S44 * S6 + 2 * C6 * SS4))) + (\
                          I4 * (-(C4 * C66 * S33 * S55) + 2 * C3 * C44 * C5 * S22 * S66 - 4 * C4 * C5 * S4 * S66 + \
                                2 * C5 * S44 * S66 + 4 * C3 * S3 * S4 * S5 * S66 + \
                                2 * C2 * S2 * (\
                                    C3 * C66 * (1 + CC5) * S44 - S3 * (C66 * S4 * S55 + 2 * C4 * S5 * S66)) + \
                                2 * CC3 * (-(C5 * S44 * S66) - C66 * SS5) + \
                                CC2 * (C4 * C66 * S33 * S55 - 8 * C3 * C6 * S3 * S4 * S5 * S6 + 4 * C4 * C5 * S4 * S66 - \
                                       2 * C66 * SS5 + 2 * CC3 * (2 * C5 * C6 * S44 * S6 + C66 * SS5)))) / 8.;
  M[1] = \
         M[7] = (CC4 * I11 * S2 * S33) / 2. - (I19 * S2 * S33) / 2. - \
                C2 * I23 * S3 * S4 + (C2 * I11 * S3 * S44) / 2. + \
                I9 * (-(C2 * (1 + CC5) * CC6 * S3 * S44) / \
                      2. + (CC6 * S2 * ((2 - CC5 - CC4 * (1 + CC5)) * S33 - C4 * CC3 * S55)) / 2.) + (\
                          I12 * (C2 * CC5 * S3 * S44 + S2 * ((1 + CC4) * CC5 * S33 + C4 * CC3 * S55))) / 2. + \
                C2 * I24 * (-(C3 * S5 * S6) + S3 * (-(C6 * S4) - C4 * C5 * S6)) + \
                I6 * (S2 * (C6 * S33 - C6 * CC4 * S33 + (C5 * S33 * S44 * S6) / 2. + C33 * S4 * S5 * S6) + \
                      C2 * (-(C3 * C4 * S5 * S6) + S3 * (-(C6 * S44) - C44 * C5 * S6))) + (\
                          I5 * S55 * (C2 * C3 * S4 - C4 * S2 * SS3)) / \
                8. + (I4 * (\
                            C2 * (-2 * C44 * C5 * S3 * S66 + C3 * (-(C66 * S4 * S55) - 2 * C4 * S5 * S66)) + \
                            S2 * (-S33 + 4 * C33 * C6 * S4 * S5 * S6 + C5 * S33 * S44 * S66 + \
                                  C4 * C66 * S55 * SS3))) / 8.;
  M[2] = M[14] = \
                 C2 * CC4 * I11 + C2 * CC4 * CC5 * I12 + C2 * CC5 * I14 + C2 * I8 + \
                 C2 * (1 - CC4 * (1 + CC5)) * CC6 * I9 + \
                 C3 * I23 * S2 * S4 - (C3 * I17 * S2 * S44) / 2. + (I5 * S2 * (-(C3 * CC5 * S44) + S3 * S4 * S55)) / \
                 8. + I24 * S2 * (-(S3 * S5 * S6) + C3 * (C6 * S4 + C4 * C5 * S6)) + (\
                     I4 * (2 * C2 * (-C66 + C66 * CC5 + 2 * C5 * C6 * S44 * S6) + \
                           S2 * (C3 * (C66 * (1 + CC5) * S44 + 4 * C44 * C5 * C6 * S6) - \
                                 S3 * (C66 * S4 * S55 + 4 * C4 * C6 * S5 * S6)))) / 8. + \
                 I6 * (S2 * (-(C4 * S3 * S5 * S6) + C3 * (2 * C4 * C6 * S4 + C44 * C5 * S6)) + \
                       C2 * (C5 * S44 * S6 + 2 * C6 * SS4));
  M[3] = \
         M[21] = -(CC5 * I10 * S2 * S3) + \
                 C4 * I23 * S2 * S3 + (I7 * S2 * S3) / 8. + (I5 * (C2 * S4 * S55 + S2 * (-S3 + C3 * C4 * S55))) / \
                 8. + I24 * S2 * S3 * (C4 * C6 - C5 * S4 * S6) + \
                 I6 * (-(C2 * C4 * S5 * S6) + S2 * (2 * C6 * S3 + C3 * S4 * S5 * S6)) + (\
                     I4 * (-(C2 * (C66 * S4 * S55 + 4 * C4 * C6 * S5 * S6)) + \
                           S2 * (C3 * (-(C4 * C66 * S55) + 4 * C6 * S4 * S5 * S6) + 2 * C66 * S3 * SS5))) / 8.;
  M[4] = \
         M[28] = (I3 * (C2 * C4 - C3 * S2 * S4)) / 4. + \
                 I24 * S2 * (C3 * C5 * S6 - C4 * S3 * S5 * S6) + \
                 I6 * (C2 * C5 * S4 * S6 + S2 * (C3 * C4 * C5 * S6 - S3 * S5 * S6)) + (\
                     I4 * (C2 * (-(C4 * C66) + 2 * C5 * C6 * S4 * S6) + \
                           S2 * (-2 * C6 * S3 * S5 * S6 + C3 * (C66 * S4 + C4 * C5 * S66)))) / 4.;
  M[5] = \
         M[35] = (I1 * (C2 * S4 * S5 + S2 * (C5 * S3 + C3 * C4 * S5))) / 2. + \
                 I6 * (C2 * C6 * S4 * S5 + S2 * (C5 * C6 * S3 + C3 * C4 * C6 * S5)) + \
                 I24 * S2 * (C3 * C6 * S5 + S3 * (C4 * C5 * C6 - S4 * S6));
  M[6] = M[42] = \
                 I2 * (C2 * (C4 * C6 - C5 * S4 * S6) + S2 * (S3 * S5 * S6 + C3 * (-(C6 * S4) - C4 * C5 * S6)));

  M[8] = Im2 + \
         I13 + (1 - CC3 * (1 + CC4)) * CC5 * I14 + CC4 * I15 + CC3 * I16 + CC3 * CC4 * I17 + \
         2 * C4 * I23 + \
         I24 * (2 * C4 * C6 - 2 * C5 * S4 * S6) + (\
             I4 * (-C66 + C66 * CC4 * (1 + CC5) + \
                   CC3 * (2 * C66 - C66 * CC5 - C66 * CC4 * (1 + CC5)) + (C4 * C66 * S33 * S55) / \
                   2. - S33 * S4 * S5 * S66 - C5 * S44 * S66 * SS3)) / \
         4. + (I5 * (-(C4 * S33 * S55) + C55 * SS4)) / 8. + \
         I6 * (2 * C6 * CC4 + S4 * (-2 * C4 * C5 - S33 * S5) * S6 + CC3 * (C5 * S44 * S6 + 2 * C6 * SS4));

  M[9] = \
         M[15] = -(I23 * S3 * S4) + (I11 * S3 * S44) / 2. - (CC5 * I14 * S3 * S44) / \
                 2. - (CC6 * I9 * S3 * S44) / 2. + (C3 * I5 * S4 * S55) / 8. - \
                 I24 * (C3 * S5 * S6 + S3 * (C6 * S4 + C4 * C5 * S6)) - \
                 I6 * (C3 * C4 * S5 * S6 + S3 * (C6 * S44 + C44 * C5 * S6)) - (\
                     I4 * (S3 * (C66 * CC5 * S44 + 2 * C44 * C5 * S66) + C3 * (C66 * S4 * S55 + 2 * C4 * S5 * S66))) / \
                 8.;

  M[10] = M[22] = \
                  C3 * C4 * I23 + (C3 * I7) / 8. + (I12 * (C3 * C55 - C4 * S3 * S55)) / 2. + \
                  C3 * I24 * (C4 * C6 - C5 * S4 * S6) + \
                  I6 * (2 * C3 * C6 - S3 * S4 * S5 * S6) + (\
                      I4 * (C3 * (1 + 2 * C66 - 4 * CC5 * CC6) + 2 * S3 * (C4 * CC6 * S55 - 2 * C6 * S4 * S5 * S6))) / \
                  8.;

  M[11] = \
          M[29] = (I3 * S3 * S4) / 4. - I6 * (C4 * C5 * S3 * S6 + C3 * S5 * S6) - \
                  I24 * (C5 * S3 * S6 + C3 * C4 * S5 * S6) - (\
                      I4 * (2 * C3 * C6 * S5 * S6 + S3 * (C66 * S4 + C4 * C5 * S66))) / 4.;

  M[12] = \
          M[36] = (I1 * (C3 * C5 - C4 * S3 * S5)) / 2. + I6 * (C3 * C5 * C6 - C4 * C6 * S3 * S5) + \
                  I24 * (-(C6 * S3 * S5) + C3 * (C4 * C5 * C6 - S4 * S6));
  \

  M[13] = \
          M[43] = I2 * (C3 * S5 * S6 + S3 * (C6 * S4 + C4 * C5 * S6));
  \


  M[16] = Im3 + I8 + \
          CC5 * I10 + CC4 * I11 + CC4 * CC5 * I12 + (1 - CC4 * (1 + CC5)) * CC6 * I9 + \
          I6 * (C5 * S44 * S6 + 2 * C6 * SS4) + (I4 * (2 * C5 * C6 * S44 * S6 - C66 * SS5)) / 4.;

  M[17] = M[23] = (I5 * S4 * S55) / 8. - \
                  C4 * I6 * S5 * S6 - (I4 * (C66 * S4 * S55 + 4 * C4 * C6 * S5 * S6)) / 8.;

  M[18] = M[30] = (C4 * I3) / 4. + \
                  C5 * I6 * S4 * S6 + (I4 * (-(C4 * C66) + 2 * C5 * C6 * S4 * S6)) / 4.;

  M[19] = M[37] = ((I1 + 2 * C6 * I6) * S4 * S5) / 2.;

  M[20] = M[44] = I2 * (C4 * C6 - C5 * S4 * S6);

  M[24] = Im4 + (I7 + C55 * I5 + 16 * C6 * I6 +  2 * C66 * I4 * SS5) / 8.;

  M[25] = M[31] = -((C6 * I4 + 2 * I6) * S5 * S6) / 2.;

  M[26] = M[38] = (C5 * (I1 + 2 * C6 * I6)) / 2.;

  M[27] = M[45] = I2 * S5 * S6;

  M[32] = Im5 + (I3 - C66 * I4) / 4.;

  M[33] = M[39] = 0;

  M[34] = M[46] = C6 * I2;

  M[40] = Im6 + I1 / 2.;

  M[41] = M[47] = 0;

  M[48] = Im7 + I2;


}
